Class: Code::Object
- Extended by:
- Concerns::Shared
- Includes:
- Concerns::Shared
- Defined in:
- lib/code/object.rb,
lib/code/object/code.rb,
lib/code/object/date.rb,
lib/code/object/html.rb,
lib/code/object/http.rb,
lib/code/object/json.rb,
lib/code/object/list.rb,
lib/code/object/time.rb,
lib/code/object/class.rb,
lib/code/object/range.rb,
lib/code/object/global.rb,
lib/code/object/string.rb,
lib/code/object/boolean.rb,
lib/code/object/context.rb,
lib/code/object/decimal.rb,
lib/code/object/integer.rb,
lib/code/object/nothing.rb,
lib/code/object/duration.rb,
lib/code/object/function.rb,
lib/code/object/parameter.rb,
lib/code/object/dictionary.rb,
lib/code/object/identifier_list.rb
Direct Known Subclasses
Boolean, Class, Code, Date, Decimal, Dictionary, Duration, Function, Global, Html, Http, Integer, Json, List, Nothing, Parameter, Range, String, Time
Defined Under Namespace
Classes: Boolean, Class, Code, Context, Date, Decimal, Dictionary, Duration, Function, Global, Html, Http, IdentifierList, Integer, Json, List, Nothing, Parameter, Range, String, Time
Constant Summary collapse
Instance Attribute Summary
Attributes included from Concerns::Shared
Class Method Summary collapse
Instance Method Summary collapse
- #code_new ⇒ Object
-
#initialize ⇒ Object
constructor
A new instance of Object.
- #name ⇒ Object
Methods included from Concerns::Shared
<=>, ==, as_json, call, code_and, code_as_json, code_deep_duplicate, code_different, code_duplicate, code_equal_equal, code_equal_equal_equal, code_exclamation_point, code_exclusive_range, code_falsy?, code_fetch, code_fetch, code_get, code_get, code_inclusive_range, code_inspect, code_name, code_or, code_self, code_set, code_set, code_to_boolean, code_to_class, code_to_date, code_to_decimal, code_to_dictionary, code_to_duration, code_to_integer, code_to_json, code_to_list, code_to_nothing, code_to_parameter, code_to_range, code_to_string, code_to_time, code_truthy?, eql?, falsy?, hash, inspect, multi_fetch, nothing?, sig, succ, to_code, to_json, to_s, truthy?
Constructor Details
#initialize ⇒ Object
Returns a new instance of Object.
18 19 |
# File 'lib/code/object.rb', line 18 def initialize(...) end |
Class Method Details
.code_new ⇒ Object
33 34 35 |
# File 'lib/code/object.rb', line 33 def self.code_new(*) new(*) end |
.maybe ⇒ Object
21 22 23 |
# File 'lib/code/object.rb', line 21 def self.maybe Type::Maybe.new(self) end |